/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

BookmarkTools.namespace("encoding");
 
BookmarkTools.encoding.ASCII85 = {

	pow85 : [52200625, 614125, 7225, 85, 1],

	wput : function(tuple, bytes, out){
		switch (bytes) {
			case 4:
				out.push(((tuple >> 24) & 0xff));
				out.push(((tuple >> 16) & 0xff));
				out.push(((tuple >>  8) & 0xff));
				out.push((tuple & 0xff));
				break;
			case 3:
				out.push(((tuple >> 24) & 0xff));
				out.push(((tuple >> 16) & 0xff));
				out.push(((tuple >>  8) & 0xff));
				break;
			case 2:
				out.push(((tuple >> 24) & 0xff));
				out.push(((tuple >> 16) & 0xff));
				break;
			case 1:
				out.push(((tuple >> 24) & 0xff));
				break;
		}
	},

	/*
	  *	Encodes the given string (utf-8) to ascii85.
	  *	@ param	string to encode
	  *	@return	ascii85 encoded byte array
	  */
	encode : function(value){
		var b = StringUtils.bytesOf(value), b0, b1, b2, b3, b4, a, r;
		var mod = 4 - b.length % 4;
		
		for(var i = 0 ;i < mod; i++)
			b.push(0);
		
		var out = [];
		for(var i = 0, l = b.length ;i < l; i += 4){
			a = (b[i] << 24) + (b[i + 1] << 16) + (b[i + 2] << 8) + b[i + 3];
			r = a % this.pow85[0];
			b0 = (a - r) / this.pow85[0];
			a = r;
			r = a % this.pow85[1];
			b1 = (a - r) / this.pow85[1];
			a = r;
			r = a % this.pow85[2];
			b2 = (a - r) / this.pow85[2];
			a = r;
			r = a % this.pow85[3];
			b3 = (a - r) / this.pow85[3];
			b4 = r;
			
			out.push((b0 + 33));
			out.push((b1 + 33));
			out.push((b2 + 33));
			out.push((b3 + 33));
			out.push((b4 + 33));
		}
		
		// add eos '~>'
		out.push(126);
		out.push(62);
		
		return out;
	},
		
	/*
	  *	Decodes the given ascii85 byte array.
	  *	@ param	encoded byte array
	  *	@return	decoded byte array
	  */
	decode : function(array){
		var tuple = 0, c, count = 0, out = [];
	
		for(var i = 0, l = array.length ;i < l; i++){
			c = array[i];
			switch (c) {
				default:
					if (c < 33 || c > 117) {
						throw "ERROR: Bad characters found!";
					}
					tuple += (c - 33) * this.pow85[count++];
					if (count == 5) {
						this.wput(tuple, 4, out);
						count = 0;
						tuple = 0;
					}
					break;
				case 122:
					if (count != 0) {
						throw "ERROR: Character z inside tuple!";
					}
					out.push(0);
					out.push(0);
					out.push(0);
					out.push(0);
					break;
				case 126:
					c = array[i + 1];
					if (c == 62) {
						if (count > 0) {
							count--;
							tuple += this.pow85[count];
							this.wput(tuple, count, out);
						}
					}
					return out;
				case 10: case 13: case 9: case 32:
				case 0: case 12: case 8: case 0177:
					break;
			}
		}
	}

};
